//  Prior Art:
//    https://playground.arduino.cc/Code/CapacitiveSensor
//    https://www.arduino.cc/en/tutorial/midi
//    https://www.instructables.com/id/Send-and-Receive-MIDI-with-Arduino/
// * This program is free software; you can redistribute it and/or modify
// * it under the terms of the GNU General Public License as published by
// * the Free Software Foundation; either version 3 of the License, or
// * (at your option) any later version.
//  0-Coast TRS Midi PinOut:
//     Tip - serial TX (Pin 1) through 220Ω resistor
//     Ring - GND 
//     Sleeve - +5v through 220Ω resistor
//  Midi.org specifies that the O-Coast uses a TRS Midi A setup-as should Line 6
//    littleBits, Korg, IK Multimedia and inMusic (Akai).  Arturia, Novation and 
//    Music1010 are all noted as type B.  THIS IS NOT SET IN STONE-YOU MAY
//    NEED TO EXPERIMENT TO ENSURE COMPATIBILITY WITH YOUR EQUIPMENT.
//  Middle C = midi note 60, defaults are E3 (52)-F4 (77)
//  Octave will default to 4 using iOS standard

#include "pins_arduino.h" // Arduino pre-1.0 needs this

#define octaveUp 30
#define octaveDown 31
#define bendDown 29
#define bendUp 28

int noteValue, velocity, octave, detect, midiValue, lastRead, minCap, maxCap, bendUpDetect, bendDownDetect, bendAmount;
void setup(){
  //note keys
  for (int thisPin = 2; thisPin<=27;thisPin++){
    pinMode(thisPin,INPUT);
  }
  //octave keys
  pinMode(octaveUp, INPUT);
  pinMode(octaveDown, INPUT);
  pinMode(bendUp, INPUT);
  pinMode(bendDown, INPUT);
  lastRead = 0;
  minCap = 3;
  maxCap = 12;
  octave = 4;
  Serial.begin(31250);
  midiCommand(0x7b,0x00, 0x00);//shut'em all off  
}
void loop(){
  //kill last note if released
  detect = readCapacitivePin(lastRead);
  if (detect<minCap) {midiCommand(0x80, keyValue(lastRead),0x00);}
  //octave up or down
  if (readCapacitivePin(octaveUp)>minCap) {octave=octave<8?octave+1:octave;}
  if (readCapacitivePin(octaveDown)>minCap) {octave=octave>-1?octave-1:octave;}
  //pitch bend
  bendUpDetect = readCapacitivePin(bendUp);
  bendDownDetect = readCapacitivePin(bendDown);
  if (bendUpDetect>minCap or bendDownDetect>minCap){
    if (bendUpDetect>bendDownDetect){
        bendAmount = 64 + (bendUpDetect *(maxCap-minCap));
        midiCommand(0xE0, 0, bendAmount);
      }
     if (bendUpDetect<bendDownDetect){
        bendAmount = 64 - (bendDownDetect*(maxCap-minCap));
        midiCommand(0xE0, 0, bendAmount);
      }
    }else{
       midiCommand(0xE0, 0, 64);
    }
  //main play loop
  for (int key = 28; key >=2; key--){//mono with high-note priority
    detect = readCapacitivePin (key);
    if (detect >= minCap){
      velocity = map(detect, minCap, maxCap, 1, 127);
      midiCommand(0x90, keyValue(key), velocity);
      lastRead = key;
      break;
    }
  }

}

int keyValue (int pinOrValue){
 midiValue = (octave*12) + (pinOrValue+2); 
 return midiValue;
}

void midiCommand(int cmd, int pitch, int velocity) {
  Serial.write(cmd);
  Serial.write(pitch);
  Serial.write(velocity);
}
uint8_t readCapacitivePin(int pinToMeasure) {
  // Variables used to translate from Arduino to AVR pin naming
  volatile uint8_t* port;
  volatile uint8_t* ddr;
  volatile uint8_t* pin;
  // Here we translate the input pin number from
  //  Arduino pin number to the AVR PORT, PIN, DDR,
  //  and which bit of those registers we care about.
  byte bitmask;
  port = portOutputRegister(digitalPinToPort(pinToMeasure));
  ddr = portModeRegister(digitalPinToPort(pinToMeasure));
  bitmask = digitalPinToBitMask(pinToMeasure);
  pin = portInputRegister(digitalPinToPort(pinToMeasure));
  // Discharge the pin first by setting it low and output
  *port &= ~(bitmask);
  *ddr  |= bitmask;
  delay(1);
  uint8_t SREG_old = SREG; //back up the AVR Status Register
  // Prevent the timer IRQ from disturbing our measurement
  noInterrupts();
  // Make the pin an input with the internal pull-up on
  *ddr &= ~(bitmask);
  *port |= bitmask;

  // Now see how long the pin to get pulled up. This manual unrolling of the loop
  // decreases the number of hardware cycles between each read of the pin,
  // thus increasing sensitivity.
  uint8_t cycles = 17;
  if (*pin & bitmask) { cycles =  0;}
  else if (*pin & bitmask) { cycles =  1;}
  else if (*pin & bitmask) { cycles =  2;}
  else if (*pin & bitmask) { cycles =  3;}
  else if (*pin & bitmask) { cycles =  4;}
  else if (*pin & bitmask) { cycles =  5;}
  else if (*pin & bitmask) { cycles =  6;}
  else if (*pin & bitmask) { cycles =  7;}
  else if (*pin & bitmask) { cycles =  8;}
  else if (*pin & bitmask) { cycles =  9;}
  else if (*pin & bitmask) { cycles = 10;}
  else if (*pin & bitmask) { cycles = 11;}
  else if (*pin & bitmask) { cycles = 12;}
  else if (*pin & bitmask) { cycles = 13;}
  else if (*pin & bitmask) { cycles = 14;}
  else if (*pin & bitmask) { cycles = 15;}
  else if (*pin & bitmask) { cycles = 16;}

  // End of timing-critical section; turn interrupts back on if they were on before, or leave them off if they were off before
  SREG = SREG_old;

  // Discharge the pin again by setting it low and output
  //  It's important to leave the pins low if you want to 
  //  be able to touch more than 1 sensor at a time - if
  //  the sensor is left pulled high, when you touch
  //  two sensors, your body will transfer the charge between
  //  sensors.
  *port &= ~(bitmask);
  *ddr  |= bitmask;

  return cycles;
}
